package ru.myitschool.work.ui.login

import android.content.Context
import android.util.Log
import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import com.google.gson.GsonBuilder
import createAuthHeader
import createRetrofit
import dagger.hilt.android.lifecycle.HiltViewModel
import dagger.hilt.android.qualifiers.ApplicationContext
import kotlinx.coroutines.flow.MutableStateFlow
import kotlinx.coroutines.flow.asStateFlow
import kotlinx.coroutines.flow.distinctUntilChanged
import kotlinx.coroutines.flow.update
import kotlinx.coroutines.launch
import retrofit2.HttpException
import ru.myitschool.work.data.remote.LoginApi
import ru.myitschool.work.data.remote.ErrorDto
import ru.myitschool.work.di.AppModule
import javax.inject.Inject

@HiltViewModel
class LoginViewModel @Inject constructor(
    @ApplicationContext private val context: Context,
    private val api: LoginApi,
    private val dataStoreManager: AppModule.DataStoreManager
) : ViewModel() {
    private val _state = MutableStateFlow(LoginState())
    val state = _state.asStateFlow()

    private val _savedUsername = MutableStateFlow<String?>(null)
    val savedUsername = _savedUsername.asStateFlow()

    private val retrofit = createRetrofit("your_username", "your_password")
    val loginApi = retrofit.create(LoginApi::class.java)

    private val _stateP = MutableStateFlow(PasswordState())
    val stateP = _stateP.asStateFlow()

    private val _savedPassword = MutableStateFlow<String?>(null)
    val savedPassword = _savedPassword.asStateFlow()

    fun initialize() {
        viewModelScope.launch {
            dataStoreManager.lastUsername.distinctUntilChanged().collect { lastUsername ->
                dataStoreManager.lastPassword.distinctUntilChanged().collect { lastpassword ->
                    if (lastUsername.isNotEmpty() && lastpassword.isNotEmpty()) {
                        _savedUsername.update { lastUsername }
                        _savedPassword.update { lastpassword }
                    } else {
                        _savedUsername.update { null }
                        _savedPassword.update { null }
                        _state.update { LoginState() }
                        _stateP.update { PasswordState() }
                    }
                }
            }
        }
    }

    fun tryLogin(username: String, password : String, onSuccess: () -> Unit) {
        viewModelScope.launch {
            try {
                val authHeader = createAuthHeader(username = username, password = password)
                val resp = loginApi.auth(authHeader, username)
                if (resp.code() != 200) {
                    throw HttpException(resp)
                }
                Log.d("LoginViewModel", "Login success for $username")
                dataStoreManager.setLastUsername(username)
                dataStoreManager.setLastPassword(password)
                onSuccess()
            } catch (httpExc: HttpException) {
                Log.e("LoginViewModel", "Login failed for $username", httpExc)
                _state.update { it.copy(error = "Login failed for: $username") }
                try {
                    httpExc.response()?.errorBody()?.string()?.let { errorString ->
                        val gson = GsonBuilder().create()
                        val errorDto = gson.fromJson(errorString, ErrorDto::class.java)
                        _state.update { it.copy(error = errorDto.error) }
                    }
                } catch (e: Exception) {
                    _state.update { it.copy(error = httpExc.message()) }
                }
            } catch (e: Exception) {
                Log.e("LoginViewModel", "Login failed for $username", e)
               _state.update { it.copy(error = "Unknown error: ${e.message}") }
            }
        }
    }

    fun onUsernameChanged(username: String) =
        _state.update { it.copy(isLoginEnabled = isUsernameValid(username), error = null) }

    companion object {
        fun isUsernameValid(username: String): Boolean {
            if (username.isEmpty() || username.length < 3 || username.first().isDigit()) {
                return false
            }
            return username.all { it.isLetterOrDigit() && it.isAsciiPrintable() }
        }

        private fun Char.isAsciiPrintable(): Boolean {
            return this.code in 32..126
        }
    }
}
